--[[
Name: TreatOrTreat
Revision: @project-revision@
Author(s): falconindy
Description: Warnings for targets and recipient rogues of Tricks of the Trade.
Inspired by: SimpleMD and Misdrection Helper
]]

local ClassColor = {
	["DEATHKNIGHT"]		= "|cffc41f3b",
	["DRUID"]		= "|cffff7d0a",
	["HUNTER"]		= "|cffabd473",
	["MAGE"]		= "|cff69ccf0",
	["PALADIN"]		= "|cfff58cba",
	["PRIEST"]		= "|cffffffff",
	["ROGUE"]		= "|cfffff569",
	["SHAMAN"]		= "|cff2459ff",
	["WARLOCK"]		= "|cff9482c9",
	["WARRIOR"]		= "|cffc79c6e",
}

local UnitDetailedThreatSituation = UnitDetailedThreatSituation
local UnitExists = UnitExists
local mfloor = math.floor
local damageMod = 1.15

--Addon Declaration
ToT = LibStub("AceAddon-3.0"):NewAddon("TrickOrTreat", "AceEvent-3.0", "AceConsole-3.0", "LibSink-2.0")

--LDB Declaration
--local frame	= CreateFrame("Button", "TrickOrTreat")
local ldb_tip
local ldb_obj	= LibStub("LibDataBroker-1.1"):NewDataObject("TrickOrTreat", {
	icon	= "Interface\\Icons\\Ability_Rogue_TricksOftheTrade.png",
	label	= "TrickOrTreat",
})


--LDB Methods
local function set_ldb_obj_text(msg)
	if msg == "TRICKS_APPLIED" then
		text = "|cffffd700Tricks on: " .. ClassColor[select(2, UnitClass(last_tricked))] .. last_tricked .. "|r"
	elseif msg == "TRICKS_EXPIRED" then
		text = "|cffffffff" .. dmgCounter_self .. "|r/|cffffffff" .. extra_dmg .. "|r" 
	elseif msg == "NO_PARTY" then
		text = "|cffbbbbbbInactive|r"
	else
		text = "|cffffd700No Target|r"
	end

	ldb_obj.text = text
end

--function ldb_obj.OnClick(_, which)
	--config goes here!
--end

function ldb_obj.OnTooltipShow(tip)
	tip:AddLine("TrickOrTreat:")
	tip:AddLine("|cff8888eeA Tricks of the Trade tracker|r")
	tip:AddLine(" ")
		
	if (GetNumRaidMembers() > 0) or (GetNumPartyMembers() > 0) then
		if last_tricked ~= nil then tricks_target = ClassColor[select(2, UnitClass(last_tricked))] .. last_tricked end

		tip:AddDoubleLine("Last Tricked:",  tricks_target or "|cffffffffN/A" .. "|r")
		tip:AddDoubleLine("Damage dealt by self:", "|cffffffff" .. dmgCounter_self .. "|r")
		tip:AddDoubleLine("Extra damage by target:", "|cffffffff" .. extra_dmg .. "|r")
	else
		tip:AddLine("|cffbbbbbbInactive: Not in a party or raid.")
	end
end

--Initial setup
function ToT:OnInitialize()
	self:SetSinkStorage({ ["sink20OutputSink"] = "RaidWarning" })
	pName = UnitName("player")
end

function ToT:OnEnable()
	self:RegisterEvent("RAID_ROSTER_UPDATE", "CheckForParty")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "CheckForParty")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "CheckForParty")
	self:RegisterEvent("GLYPH_ADDED", 'ScanGlyph')
	self:RegisterEvent("GLYPH_REMOVED", 'ScanGlyph')
	self:RegisterEvent("GLYPH_UPDATED", 'ScanGlyph')

	dmgCounter_self = 0
	dmgCounter_target = 0
	extra_dmg = 0
	parse_dmg_self = false
	parse_dmg_target = false
	last_tricked = nil
	
	set_ldb_obj_text()
	ScanGlyph()
end

function ToT:CheckForParty()
	if (GetNumRaidMembers() > 0) or (GetNumPartyMembers() > 0) then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		tricks_target = nil
	else
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		last_tricked = nil
		set_ldb_obj_text("NO_PARTY")
	end
end

function ToT:COMBAT_LOG_EVENT_UNFILTERED(event, _, subevent, _,sourceName, _, _, destName, _, spellID, spellName, _, arg12)
	if spellID == 57933 then
		if subevent == "SPELL_AURA_APPLIED" and sourceName == pName then
			parse_dmg_target = true
			local themessage = "Trick or Treat!"
			if UnitExists("target") then --To avoid issues when Tricking with FoK and no target
				local isTanking,_,threatpct,_,threatvalue = UnitDetailedThreatSituation(destName, "target");
				if threatvalue == nil then threatvalue, threatpct = 0, 0 end
				threatpct = threatpct * 1.1
				local rawtotank = threatvalue * (100 / threatpct - 1)
				if not isTanking and (threatpct > 50 or rawtotank <= 20000) then
					threatpct = tostring(mfloor(threatpct))
					themessage = themessage.." Careful! You're at "..threatpct.."% threat on "..UnitName("target").."."
				end
			end
			SendChatMessage(themessage, "WHISPER", nil, destName)
		elseif subevent == "SPELL_AURA_APPLIED" and destName == pName then
			self:Pour(sourceName.." tricked you!")
			PlaySoundFile("Interface\\AddOns\\TrickOrTreat\\sounds\\alert.mp3")
		end
	end
	if spellID == 57934 then
		if subevent == "SPELL_CAST_SUCCESS" and sourceName == pName then
			last_tricked = destName
			parse_dmg_self = true
			self:Pour("|cffff6600[Trick or Treat] "..ClassColor[select(2, UnitClass(destName))]..destName.."|r")
			set_ldb_obj_text("TRICKS_APPLIED")
			dmgCounter_self = 0
			dmgCounter_target = 0
		elseif subevent == "SPELL_AURA_REMOVED" and dmgCounter_self == 0 then
			set_ldb_obj_text()
		end
	end
	if spellID == 59628 and subevent == "SPELL_AURA_REMOVED" then
		if destName == pName then
			extra_dmg = mfloor(dmgCounter_target - dmgCounter_target/damageMod)
			self:Print(dmgCounter_self.." damage dealt during Tricks. Target did "..extra_dmg.." extra.")
			SendChatMessage("Tricks gave you "..extra_dmg.." extra damage.", "WHISPER", nil, last_tricked)
			
			parse_dmg_target = false
			parse_dmg_self = false
			set_ldb_obj_text("TRICKS_EXPIRED")
		end
	end

	--DAMAGE PARSING HANDLERS
	if parse_dmg_self then
		if (subevent == "SPELL_DAMAGE" or subevent == "SPELL_PERIODIC_DAMAGE" or subevent == "RANGE_DAMAGE") and sourceName == pName then
			dmgCounter_self = self:AddDamage(dmgCounter_self, arg12, arg13)
		elseif subevent == "SWING_DAMAGE" and sourceName == pName then
			dmgCounter_self = self:AddDamage(dmgCounter_self, arg9, arg10)
		end
	end

	if parse_dmg_target then
		if (subevent == "SPELL_DAMAGE" or subevent == "SPELL_PERIODIC_DAMAGE" or subevent == "RANGE_DAMAGE") and sourceName == last_tricked then
			dmgCounter_target = self:AddDamage(dmgCounter_target, arg12, arg13)
		elseif subevent == "SWING_DAMAGE" and sourceName == pName then
			dmgCounter_target = self:AddDamage(dmgCounter_target, arg9, arg10)
		end
	end

end

--Helper functions
function ToT:AddDamage(counter, damage, overkill)
	return counter + damage - overkill
end

function ToT:ScanGlyph()
	for i = 1, 6 do
		local _, _, glyphSpell = GetGlyphSocketInfo(i)
		
		if glyphSpell == 63898 then
			damageMod = 1.25
			return
		end
	end
end

--Relevant SpellIDs (dummy reference ftw)
--ID	event	src	dest	desc	
--57934	c_suc	player	target	PRIMER ON SELF
--57934	a_app	player	player	PRIMER ON SELF/TARGET
--57934	a_rem	player	target	PRIMER ON SELF
--59628	a_app	player	player	THREAT TRANSFER ON SELF
--59628	a_rem	player	player	THREAT TRANSFER ON SELF
--57933	a_app	player	target	THREAT TRANSFER ON TARGET
--57933	a_rem	player	target	THREAT TRANSFER ON TARGET
